<?php

namespace Drupal\Tests\watchdog_mailer\Functional;

/**
 * This class provides methods specifically for testing something.
 *
 * @group watchdog_mailer
 */
class WatchdogMailerSettingsTest extends WatchdogMailerTestBase {

  /**
   * Tests initial values.
   */
  public function testInitialValues() {
    $session = $this->assertSession();
    // Go to the settings page and check if the default settings are applied.
    $this->drupalGet('/admin/config/development/watchdog_mailer');
    $session->checkboxChecked('watchdog_mailer_enable');
    $session->fieldValueEquals('recipients_default', '');
    $session->fieldValueEquals('mail_subject', "[Watchdog Mailer ([watchdog_mailer:channel])] \"[watchdog_mailer:message]\" at [site:name] ([site:url])");
    $session->fieldValueEquals('mail_body', "[site:name] ([site:url]) reported the following:\n\n[watchdog_mailer:message]\n[watchdog_mailer:php_backtrace]\n\n---\n\nSite: [site:url]\nChannel: [watchdog_mailer:channel]\nBy: [watchdog_mailer:user:display-name] (UID: [watchdog_mailer:user], IP: [watchdog_mailer:user_ip])\nAt: [watchdog_mailer:timestamp:medium]\nReferrer: [watchdog_mailer:referrer]\n\n---\nThis message was generated by Watchdog Mailer\n[watchdog_mailer:watchdog_mailer_url]");
    $session->checkboxChecked('enabled');
    $session->fieldValueEquals('channels', '');
    $session->checkboxNotChecked('severities[rfc0]');
    $session->checkboxNotChecked('severities[rfc1]');
    $session->checkboxNotChecked('severities[rfc2]');
    $session->checkboxNotChecked('severities[rfc3]');
    $session->checkboxNotChecked('severities[rfc4]');
    $session->checkboxNotChecked('severities[rfc5]');
    $session->checkboxNotChecked('severities[rfc6]');
    $session->checkboxNotChecked('severities[rfc7]');
    $session->fieldValueEquals('recipients', '');
  }

  /**
   * Tests the basic functionality of the settings page.
   */
  public function testBasicSettingsChange() {
    $session = $this->assertSession();
    // Go to the settings page and change all the settings.
    $this->drupalGet('/admin/config/development/watchdog_mailer');
    $edit = [
      'watchdog_mailer_enable' => FALSE,
      'recipients_default' => 'foo@bar.baz,x@y.z',
      'mail_subject' => 'test_mail_subject',
      'mail_body' => 'test_mail_body',
      'enabled' => FALSE,
      'channels' => "test_channelA\ntest_channelB",
      'severities[rfc0]' => TRUE,
      'severities[rfc1]' => TRUE,
      'severities[rfc2]' => TRUE,
      'severities[rfc3]' => TRUE,
      'severities[rfc4]' => TRUE,
      'severities[rfc5]' => TRUE,
      'severities[rfc6]' => TRUE,
      'severities[rfc7]' => TRUE,
      'recipients' => 'baz@foo.bar, a@b.c',
    ];
    $this->submitForm($edit, 'Save configuration');
    // Check saved settings.
    $session->pageTextContains('The configuration options have been saved.');
    $session->checkboxNotChecked('watchdog_mailer_enable');
    $session->fieldValueEquals('recipients_default', 'foo@bar.baz,x@y.z');
    $session->fieldValueEquals('mail_subject', 'test_mail_subject');
    $session->fieldValueEquals('mail_body', 'test_mail_body');
    $session->checkboxNotChecked('enabled');
    $session->fieldValueEquals('channels', "test_channelA\ntest_channelB");
    $session->checkboxChecked('severities[rfc0]');
    $session->checkboxChecked('severities[rfc1]');
    $session->checkboxChecked('severities[rfc2]');
    $session->checkboxChecked('severities[rfc3]');
    $session->checkboxChecked('severities[rfc4]');
    $session->checkboxChecked('severities[rfc5]');
    $session->checkboxChecked('severities[rfc6]');
    $session->checkboxChecked('severities[rfc7]');
    $session->fieldValueEquals('recipients', 'baz@foo.bar,a@b.c');
    // Check configuration.
    $this->rebuildContainer();
    $watchdogMailerSettings = $this->config('watchdog_mailer.settings');
    $this->assertEquals($watchdogMailerSettings->get('watchdog_mailer_enable'), FALSE);
    $this->assertEquals($watchdogMailerSettings->get('recipients_default'), ['foo@bar.baz', 'x@y.z']);
    $this->assertEquals($watchdogMailerSettings->get('mail_subject'), 'test_mail_subject');
    $this->assertEquals($watchdogMailerSettings->get('mail_body'), 'test_mail_body');
    $this->assertEquals($watchdogMailerSettings->get('notification_objects.0.enabled'), FALSE);
    $this->assertEquals($watchdogMailerSettings->get('notification_objects.0.channels'), [
      'test_channelA', 'test_channelB',
    ]);
    $this->assertEquals($watchdogMailerSettings->get('notification_objects.0.severities'), [
      0 => 0,
      1 => 1,
      2 => 2,
      3 => 3,
      4 => 4,
      5 => 5,
      6 => 6,
      7 => 7,
    ]);
    $this->assertEquals($watchdogMailerSettings->get('notification_objects.0.recipients'), ['baz@foo.bar', 'a@b.c']);
  }

  /**
   * Test settings form error functions.
   */
  public function testSettingsFormErrors() {
    $this->drupalGet('/admin/config/development/watchdog_mailer');
    // Test the recipients_default field.
    // Test single invalid email-address.
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'recipients_default' => 'invalid_email_address',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->statusMessageContains('invalid_email_address is not a valid email address.', 'error');
    // Test multiple invalid email-address.
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'recipients_default' => 'example@example.com, foobar.baz_invalid',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->statusMessageContains('foobar.baz_invalid is not a valid email address.', 'error');
    // Test invalid comma separation.
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'recipients_default' => 'example@example.com, foobar@baz.bar,',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->statusMessageContains('Invalid comma separation within email addresses.', 'error');
    // Do the same for the recipients field.
    // Test single invalid email-address.
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'recipients' => 'invalid_email_address',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->statusMessageContains('invalid_email_address is not a valid email address.', 'error');
    // Test multiple invalid email-address.
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'recipients' => 'example@example.com, foobar.baz_invalid',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->statusMessageContains('foobar.baz_invalid is not a valid email address.', 'error');
    // Test invalid comma separation.
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'recipients_default' => 'example@example.com, foobar@baz.bar,',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->statusMessageContains('Invalid comma separation within email addresses.', 'error');
  }

}
