<?php

declare(strict_types=1);

namespace Drupal\Tests\watchdog_mailer\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the #states functionality of the Watchdog Mailer module settings form.
 *
 * @group watchdog_mailer
 */
final class SettingsFormStatesTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['watchdog_mailer'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'claro';

  /**
   * A user with admin permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->adminUser = $this->drupalCreateUser(['administer watchdog_mailer']);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Test callback.
   */
  public function testSettingsFormStates(): void {
    $this->drupalGet('/admin/config/development/watchdog_mailer');
    $page = $this->getSession()->getPage();
    $watchdog_mailer_enable = $page->findField('watchdog_mailer_enable');
    $watchdog_mailer_settings = $page->find('css', '#edit-mail-settings');
    $recipients_default = $page->findField('recipients_default');
    $mail_subject = $page->findField('mail_subject');
    $mail_body = $page->findField('mail_body');
    $enabled = $page->findField('enabled');
    $channels = $page->findField('channels');
    $severities = $page->find('css', '#edit-severities--wrapper');
    $recipients = $page->findField('recipients');
    // Test initial condition.
    $this->assertTrue($watchdog_mailer_enable->isVisible());
    $this->assertTrue($watchdog_mailer_settings->isVisible());
    $this->assertTrue($recipients_default->isVisible());
    $this->assertTrue($mail_subject->isVisible());
    $this->assertTrue($mail_body->isVisible());
    $this->assertTrue($enabled->isVisible());
    $this->assertTrue($channels->isVisible());
    $this->assertTrue($severities->isVisible());
    $this->assertTrue($recipients->isVisible());
    // Disable watchdog mailer functionality.
    $watchdog_mailer_enable->uncheck();
    $this->assertTrue($watchdog_mailer_enable->isVisible());
    $this->assertFalse($watchdog_mailer_settings->isVisible());
    $this->assertFalse($recipients_default->isVisible());
    $this->assertFalse($mail_subject->isVisible());
    $this->assertFalse($mail_body->isVisible());
    $this->assertFalse($enabled->isVisible());
    $this->assertFalse($channels->isVisible());
    $this->assertFalse($severities->isVisible());
    $this->assertFalse($recipients->isVisible());
  }

}
